@@ -3,8 +3,16 @@
 use crate::box_coll::ErgoBoxCandidates;
 use crate::box_coll::ErgoBoxes;
 use crate::data_input::DataInputs;
+use crate::error_conversion::to_js;
 use crate::input::{Inputs, UnsignedInputs};
+use crate::json::TransactionJsonEip12;
+use crate::json::UnsignedTransactionJsonEip12;
 use ergo_lib::chain;
+use ergo_lib::chain::transaction::distinct_token_ids;
+use ergo_lib::ergotree_ir::chain::base16_bytes::Base16DecodedBytes;
+use ergo_lib::ergotree_ir::chain::base16_bytes::Base16EncodedBytes;
+use ergo_lib::ergotree_ir::chain::digest32::Digest32;
+use js_sys::Uint8Array;
 use std::convert::TryFrom;
 use std::convert::TryInto;
 use wasm_bindgen::prelude::*;
@@ -26,15 +34,14 @@ impl TxId {
 
     /// get the tx id as bytes
     pub fn to_str(&self) -> String {
-        let base16_bytes = ergo_lib::chain::Base16EncodedBytes::new(self.0 .0 .0.as_ref());
+        let base16_bytes = Base16EncodedBytes::new(self.0 .0 .0.as_ref());
         base16_bytes.into()
     }
 
     /// convert a hex string into a TxId
     #[allow(clippy::should_implement_trait)]
     pub fn from_str(s: &str) -> Result<TxId, JsValue> {
-        let bytes = ergo_lib::chain::Base16DecodedBytes::try_from(s.to_string())
-            .map_err(|e| JsValue::from_str(&format!("{}", e)))?;
+        let bytes = Base16DecodedBytes::try_from(s.to_string()).map_err(to_js)?;
 
         bytes
             .try_into()
@@ -42,7 +49,7 @@ impl TxId {
             .map_err(|_e| {
                 JsValue::from_str(&format!(
                     "Expected a Vec of length {} but it was {}",
-                    chain::Digest32::SIZE,
+                    Digest32::SIZE,
                     s.len()
                 ))
             })
@@ -64,41 +71,70 @@ pub struct Transaction(chain::transaction::Transaction);
 
 #[wasm_bindgen]
 impl Transaction {
+    /// Create Transaction from UnsignedTransaction and an array of proofs in the same order as
+    /// UnsignedTransaction.inputs with empty proof indicated with empty byte array
+    pub fn from_unsigned_tx(
+        unsigned_tx: UnsignedTransaction,
+        proofs: Vec<Uint8Array>,
+    ) -> Result<Transaction, JsValue> {
+        chain::transaction::Transaction::from_unsigned_tx(
+            unsigned_tx.0,
+            proofs
+                .into_iter()
+                .map(|bytes| bytes.to_vec().into())
+                .collect(),
+        )
+        .map_err(|e| JsValue::from_str(&format!("{}", e)))
+        .map(Into::into)
+    }
+
     /// Get id for transaction
     pub fn id(&self) -> TxId {
         self.0.id().into()
     }
 
-    /// JSON representation
-    pub fn to_json(&self) -> Result<JsValue, JsValue> {
-        JsValue::from_serde(&self.0.clone()).map_err(|e| JsValue::from_str(&format!("{}", e)))
+    /// JSON representation as text (compatible with Ergo Node/Explorer API, numbers are encoded as numbers)
+    pub fn to_json(&self) -> Result<String, JsValue> {
+        serde_json::to_string_pretty(&self.0.clone())
+            .map_err(|e| JsValue::from_str(&format!("{}", e)))
     }
 
-    /// JSON representation
+    /// JSON representation according to EIP-12 https://github.com/ergoplatform/eips/pull/23
+    /// (similar to [`Self::to_json`], but as JS object with box value and token amount encoding as strings)
+    pub fn to_js_eip12(&self) -> Result<JsValue, JsValue> {
+        let tx_dapp: TransactionJsonEip12 = self.0.clone().into();
+        JsValue::from_serde(&tx_dapp).map_err(|e| JsValue::from_str(&format!("{}", e)))
+    }
+
+    /// parse from JSON
+    /// supports Ergo Node/Explorer API and box values and token amount encoded as strings
     pub fn from_json(json: &str) -> Result<Transaction, JsValue> {
-        serde_json::from_str(json)
-            .map(Self)
-            .map_err(|e| JsValue::from_str(&format!("{}", e)))
+        serde_json::from_str(json).map(Self).map_err(to_js)
     }
 
     /// Inputs for transaction
     pub fn inputs(&self) -> Inputs {
-        self.0.inputs.clone().into()
+        self.0.inputs.as_vec().clone().into()
     }
 
     /// Data inputs for transaction
     pub fn data_inputs(&self) -> DataInputs {
-        self.0.data_inputs.clone().into()
+        self.0
+            .data_inputs
+            .clone()
+            .map(|di| di.as_vec().clone())
+            .unwrap_or_default()
+            .into()
     }
 
     /// Output candidates for transaction
     pub fn output_candidates(&self) -> ErgoBoxCandidates {
-        self.0.output_candidates.clone().into()
+        self.0.output_candidates.as_vec().clone().into()
     }
 
     /// Returns ErgoBox's created from ErgoBoxCandidate's with tx id and indices
     pub fn outputs(&self) -> ErgoBoxes {
-        self.0.outputs().into()
+        self.0.outputs.clone().into()
     }
 }
 
@@ -122,30 +158,50 @@ impl UnsignedTransaction {
 
     /// Inputs for transaction
     pub fn inputs(&self) -> UnsignedInputs {
-        self.0.inputs.clone().into()
+        self.0.inputs.as_vec().clone().into()
     }
 
     /// Data inputs for transaction
     pub fn data_inputs(&self) -> DataInputs {
-        self.0.data_inputs.clone().into()
+        self.0
+            .clone()
+            .data_inputs
+            .map(|di| di.as_vec().clone())
+            .unwrap_or_default()
+            .into()
     }
 
     /// Output candidates for transaction
     pub fn output_candidates(&self) -> ErgoBoxCandidates {
-        self.0.output_candidates.clone().into()
+        self.0.output_candidates.as_vec().clone().into()
     }
 
-    /// JSON representation
-    pub fn to_json(&self) -> Result<JsValue, JsValue> {
-        JsValue::from_serde(&self.0.clone()).map_err(|e| JsValue::from_str(&format!("{}", e)))
-    }
-
-    /// JSON representation
-    pub fn from_json(json: &str) -> Result<UnsignedTransaction, JsValue> {
-        serde_json::from_str(json)
-            .map(Self)
+    /// JSON representation as text (compatible with Ergo Node/Explorer API, numbers are encoded as numbers)
+    pub fn to_json(&self) -> Result<String, JsValue> {
+        serde_json::to_string_pretty(&self.0.clone())
             .map_err(|e| JsValue::from_str(&format!("{}", e)))
     }
+
+    /// JSON representation according to EIP-12 https://github.com/ergoplatform/eips/pull/23
+    /// (similar to [`Self::to_json`], but as JS object with box value and token amount encoding as strings)
+    pub fn to_js_eip12(&self) -> Result<JsValue, JsValue> {
+        let tx_dapp: UnsignedTransactionJsonEip12 = self.0.clone().into();
+        JsValue::from_serde(&tx_dapp).map_err(|e| JsValue::from_str(&format!("{}", e)))
+    }
+
+    /// parse from JSON
+    /// supports Ergo Node/Explorer API and box values and token amount encoded as strings
+    pub fn from_json(json: &str) -> Result<UnsignedTransaction, JsValue> {
+        serde_json::from_str(json).map(Self).map_err(to_js)
+    }
+
+    /// Returns distinct token id from output_candidates as array of byte arrays
+    pub fn distinct_token_ids(&self) -> Vec<Uint8Array> {
+        distinct_token_ids(self.0.output_candidates.clone())
+            .iter()
+            .map(|id| Uint8Array::from(id.as_ref()))
+            .collect()
+    }
 }
 
 impl From<chain::transaction::unsigned::UnsignedTransaction> for UnsignedTransaction {
